#!/usr/bin/perl -w
# NAME: tidytests.pl
# AIM: VERY SPECIFIC: Read test cases
use strict;
use warnings;
use File::Basename;  # split path ($name,$dir,$ext) = fileparse($file [, qr/\.[^.]*/] )
use Cwd;
my $os = $^O;
my $perl_dir = '/home/geoff/bin';
my $PATH_SEP = '/';
my $temp_dir = '/tmp';
if ($os =~ /win/i) {
    $perl_dir = 'C:\GTools\perl';
    $temp_dir = $perl_dir;
    $PATH_SEP = "\\";
}
unshift(@INC, $perl_dir);
require 'lib_utils.pl' or die "Unable to load 'lib_utils.pl' Check paths in \@INC...\n";
require 'lib_html.pl' or die "Unable to load 'lib_html.pl' Check paths in \@INC...\n";

# log file stuff
our ($LF);
my $pgmname = $0;
if ($pgmname =~ /(\\|\/)/) {
    my @tmpsp = split(/(\\|\/)/,$pgmname);
    $pgmname = $tmpsp[-1];
}
my $outfile = $temp_dir.$PATH_SEP."temp.$pgmname.txt";
open_log($outfile);

# user variables
my $VERS = "0.0.5 2015-01-09";
my $load_log = 0;
my $in_file = '';
my $verbosity = 0;
my $out_file = '';

my $input_dir = 'input';
my $output_dir = 'temp-5';

# ### DEBUG ###
my $debug_on = 1;
my $def_file = 'F:\Projects\tidy-html5\test\testcases.txt';

### program variables
my @warnings = ();
my $cwd = cwd();

sub VERB1() { return $verbosity >= 1; }
sub VERB2() { return $verbosity >= 2; }
sub VERB5() { return $verbosity >= 5; }
sub VERB9() { return $verbosity >= 9; }

sub show_warnings($) {
    my ($val) = @_;
    if (@warnings) {
        prt( "\nGot ".scalar @warnings." WARNINGS...\n" );
        foreach my $itm (@warnings) {
           prt("$itm\n");
        }
        prt("\n");
    } else {
        prt( "\nNo warnings issued.\n\n" ) if (VERB9());
    }
}

sub pgm_exit($$) {
    my ($val,$msg) = @_;
    if (length($msg)) {
        $msg .= "\n" if (!($msg =~ /\n$/));
        prt($msg);
    }
    show_warnings($val);
    close_log($outfile,$load_log);
    exit($val);
}


sub prtw($) {
   my ($tx) = shift;
   $tx =~ s/\n$//;
   prt("$tx\n");
   push(@warnings,$tx);
}

sub get_input($) {
	my ($m) = shift;
	my $in = '';
	prt($m);
	$in = <>;
	chomp $in;
	if (uc($in) eq 'Y') {
		prt( "Got [$in] ...YES ...\n" );
	} else {
		prt( "Got [$in] ...\n" );
	}
	return $in;
}


sub get_iframe($$$) {
    my ($in,$out,$val) = @_;
    my $txt = <<EOF;
<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8">
<title>iframe testing $val</title>
</head>
<body>
<iframe align="left" width="45%" height="100%" src="$in"></iframe>
<iframe align="right" width="45%" height="100%" src="$out"></iframe>
</body>
</html>
EOF
    return $txt;
}

sub process_in_file($) {
    my ($inf) = @_;
    if (! open INF, "<$inf") {
        pgm_exit(1,"ERROR: Unable to open file [$inf]\n"); 
    }
    my @lines = <INF>;
    close INF;
    my $lncnt = scalar @lines;
    my ($name,$dir) = fileparse($inf);
    ut_fix_directory(\$dir);
    my ($in,$out);
    my ($in2,$out2);
    prt("Processing $lncnt lines, from [$inf]...\n");
    my ($line,$inc,$lnn,$len,@arr,$val,$html,$test,$res,$title,$input);
    $lnn = 0;
    my @reslines = ();
    my $resfile = $dir."tempres.txt";
    foreach $line (@lines) {
        chomp $line;
        $lnn++;
        $line = trim_all($line);
        $len = length($line);
        next if ($len == 0);
        @arr = split(/\s+/,$line);
        $len = scalar @arr;
        if ($len == 2) {
            $inc = $arr[0];
            $val = $arr[1];
            $in2 = "$input_dir/in_$inc.html";
            $out2 = "$output_dir/out_$inc.html";
            $in = $dir.$input_dir.$PATH_SEP."in_$inc.html";
            $out = $dir.$output_dir.$PATH_SEP."out_$inc.html";
            $test = $dir."temp_$inc.html";
            if ((-f $in) && (-f $out)) {
                $title = get_html_title_from_file($in);
                $html = get_iframe($in2,$out2,$inc);
                write2file($html,$test);
                prt("$lnn: $inc $val ok $test\n");
                system("start $test");
                $input = get_input("S for SAME, else different\n");
                if (($input eq 'Q')||($input eq 'q')) {
                    last;
                }
                $res = "$inc $val $title $input";
            } else {
                if (-f $in) {
                    $title = get_html_title_from_file($in);
                    if ($val == 2) {
                        prt("$lnn: $inc $val no out expected\n");
                    } else {
                        prt("$lnn: $inc $val no OUT?\n");
                    }
                    $res = "$inc $val $title";
                } else {
                    prt("$lnn: $inc $val no IN!\n");
                    $res = "$inc $val NO INPUT $in2!";
                }
            }
            push(@reslines,$res);
        } else {
            prtw("WARNING: $lnn: Did not split in 2, got $len [$line]\n");
        }
    }
    write2file(join("\n",@reslines)."\n",$resfile);
}

#########################################
### MAIN ###
parse_args(@ARGV);
process_in_file($in_file);
pgm_exit(0,"");
########################################

sub need_arg {
    my ($arg,@av) = @_;
    pgm_exit(1,"ERROR: [$arg] must have a following argument!\n") if (!@av);
}

sub parse_args {
    my (@av) = @_;
    my ($arg,$sarg);
    my $verb = VERB2();
    while (@av) {
        $arg = $av[0];
        if ($arg =~ /^-/) {
            $sarg = substr($arg,1);
            $sarg = substr($sarg,1) while ($sarg =~ /^-/);
            if (($sarg =~ /^h/i)||($sarg eq '?')) {
                give_help();
                pgm_exit(0,"Help exit(0)");
            } elsif ($sarg =~ /^v/) {
                if ($sarg =~ /^v.*(\d+)$/) {
                    $verbosity = $1;
                } else {
                    while ($sarg =~ /^v/) {
                        $verbosity++;
                        $sarg = substr($sarg,1);
                    }
                }
                $verb = VERB2();
                prt("Verbosity = $verbosity\n") if ($verb);
            } elsif ($sarg =~ /^l/) {
                if ($sarg =~ /^ll/) {
                    $load_log = 2;
                } else {
                    $load_log = 1;
                }
                prt("Set to load log at end. ($load_log)\n") if ($verb);
            } elsif ($sarg =~ /^o/) {
                need_arg(@av);
                shift @av;
                $sarg = $av[0];
                $out_file = $sarg;
                prt("Set out file to [$out_file].\n") if ($verb);
            } else {
                pgm_exit(1,"ERROR: Invalid argument [$arg]! Try -?\n");
            }
        } else {
            $in_file = $arg;
            prt("Set input to [$in_file]\n") if ($verb);
        }
        shift @av;
    }

    if ($debug_on) {
        prtw("WARNING: DEBUG is ON!\n");
        if (length($in_file) ==  0) {
            $in_file = $def_file;
            prt("Set DEFAULT input to [$in_file]\n");
        }
    }
    if (length($in_file) ==  0) {
        pgm_exit(1,"ERROR: No input files found in command!\n");
    }
    if (! -f $in_file) {
        pgm_exit(1,"ERROR: Unable to find in file [$in_file]! Check name, location...\n");
    }
}

sub give_help {
    prt("$pgmname: version $VERS\n");
    prt("Usage: $pgmname [options] in-file\n");
    prt("Options:\n");
    prt(" --help  (-h or -?) = This help, and exit 0.\n");
    prt(" --verb[n]     (-v) = Bump [or set] verbosity. def=$verbosity\n");
    prt(" --load        (-l) = Load LOG at end. ($outfile)\n");
    prt(" --out <file>  (-o) = Write output to this file.\n");
}

# eof - template.pl
